# PeerTube collector

peertube-collector is a project designed to collect and analyze WebRTC statistics from a Chromium browser and export them to a MongoDB service. This project includes a Docker setup for running the necessary services.

## Prerequisites

**Linux** based OS with the following:

### Software:
- Docker Engine Community version is required. To install Docker CE, follow the official [install instructions](https://docs.docker.com/engine/install/).

### Ports: 
#### Docker to Host:
- 4444 (Selenium)

Ports can be opened in the host machine's firewall with:
```sh
ufw allow from 172.100.0.0/16 to any port 4444
```

#### External (OPTIONAL):
These ports are actively used by selenium and the collector services. By defaut they should not be blocked by the firewall, but if so, they can be opened in the host machine's firewall.

- 50000:60000/udp (WebRTC)
  - Needed for WebRTC NAT traversal, otherwise the browser will not connect to any peer.
    The range needs to be fairly large since the port is chosen randomly by the STUN server.
- 27107/tcp (MongoDB)

Ports can be opened in the host machine's firewall with:
```sh
ufw allow 50000:60000/udp
ufw allow 27107/tcp
```
## Setup

1. Clone the repository:
    ```sh
    git clone <repository-url>
    cd peertube-collector
    ```

2. Create and configure the environment file based on the `.env.example` file:
    ```sh
    cp .env.example .env
    ```

3. Ajust the firewall settings to allow the necessary ports if needed

4. Start the Docker containers:
    ```sh
    docker compose up --abort-on-container-failure
    ```

The collector will start gathering WebRTC stats from the Selenium container and sending them to the Telegraf service.

To stop the Docker containers run: `docker compose down -v`

### Monitoring
A noVNC server is available at [http://localhost:7900](http://localhost:7900/?autoconnect=1&resize=scale&password=secret) to monitor the Selenium container. The password is `secret`.

## Components

### Docker Compose

The `docker-compose.yml` file defines the following services:
- **selenium**: Runs a Selenium standalone Chromium container.
- **telegraf**: Collects and sends metrics to the specified output.
- **collector**: Runs the main Python application to collect WebRTC stats.

### Dockerfile

The `Dockerfile` sets up the Python environment and installs the necessary dependencies to run the `main.py` script.

### Main Python Script

The `main.py` script sets up the Selenium WebDriver, collects WebRTC stats, and sends them to the Telegraf service.

### WebRTC Internals Exporter

The `webrtc-internals-exporter` directory contains a Chromium extension that collects WebRTC stats from the browser.

## Working Project Structure

```
peertube-collector/
├── requirements.txt
├── telegraf.conf
├── docker-compose.yml
├── Dockerfile
├── main.py
├── .env
└── utils/
└── webrtc-internals-exporter/
```

# Credits

- [WebRTC Internals Exporter](https://github.com/vpalmisano/webrtc-internals-exporter)
  - [WebRTC debugging with Prometheus/Grafana](https://medium.com/@vpalmisano/webrtc-debugging-with-prometheus-grafana-254b6ac71063)
- [MongoDB Docker Compose examples](https://github.com/TGITS/docker-compose-examples/tree/main/mongodb-docker-compose-examples)